import os
import sys

import maya.cmds
import maya.api.OpenMaya as OpenMaya

from PySide2 import QtCore
from PySide2 import QtWidgets

# __file__ is not defined in Maya Python plug-ins
import inspect
__file__ = inspect.getfile(lambda: None)

PARENT_DIR = os.path.dirname(os.path.abspath(__file__))
CONFIG_DIR_DIR = PARENT_DIR

VERSION_STR = "1.8.0"
MENU_NAME = "GridMarkets"
MENU_ITEM_NAME = "Open GridMarkets Plugin"
LOGGER_NAME = 'gridmarkets_maya_plugin'
PRODUCT = 'maya'
META_INFO = 'gridmarkets-maya-plugin version: ' + VERSION_STR
DEBUG_MODE = os.path.exists(os.path.join(PARENT_DIR, 'debug'))
VENDOR = 'GridMarkets Pte. Ltd.'

GM_LOGO_PATH = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "icons/gm_icon_white.png")

if sys.version_info[0] >= 3:
    import configparser
    config = configparser.ConfigParser()
    config.read(os.path.join(CONFIG_DIR_DIR, 'config.ini'))

    ENVOY_PYTHON_PATH = config.get('default', 'ENVOY_PYTHON_PATH')

    if config.has_section('submit2gm'):
        SUBMIT2GM_PATH = config.get('submit2gm', 'SUBMIT2GM_PATH')
    else:
        SUBMIT2GM_PATH = config.get('default', 'SUBMIT2GM_PATH')

else:
    import ConfigParser as configparser
    config = configparser.ConfigParser()
    config.read(os.path.join(CONFIG_DIR_DIR, 'config.ini'))

    ENVOY_PYTHON_PATH = config.get('default', 'ENVOY_PYTHON_PATH')

    if config.has_section('submit2gm'):
        SUBMIT2GM_PATH = config.get('submit2gm', 'SUBMIT2GM_PATH')
    else:
        SUBMIT2GM_PATH = config.get('default', 'SUBMIT2GM_PATH')


def evaluate_code(code, return_variables):
    # Execute the provided code
    try:
        exec (code)
    except Exception as e:
        raise e

    results = {}

    for variable in return_variables:
        results[variable] = eval(variable)

    return results


def process_events():
    try:
        from submit2gm.command_port_server_utils import process_events
        process_events()
    except ImportError:
        pass


def about_to_quit_slot():
    try:
        from submit2gm.command_port_server_utils import stop_submit2gm
        stop_submit2gm()
    except ImportError:
        pass


event_processor_timer = QtCore.QTimer()
event_processor_timer.timeout.connect(process_events)
event_processor_timer.start()

QtWidgets.QApplication.instance().aboutToQuit.connect(about_to_quit_slot)


def import_submit2gm():
    # Read config file to find submit2gm install location
    submit2gm_path = SUBMIT2GM_PATH

    # Get the location of the submit2gm python package
    submit2gm_package_path = os.path.join(os.path.dirname(submit2gm_path), 'submit2gm')

    # Import the submit2gm package
    submit2gm_init_path = os.path.join(submit2gm_package_path, "__init__.py")

    if sys.version_info[0] >= 3:
        from importlib import util as importlib_util
        spec = importlib_util.spec_from_file_location('submit2gm', submit2gm_init_path)
        module = importlib_util.module_from_spec(spec)

        sys.modules[spec.name] = module
        spec.loader.exec_module(module)

    elif 'submit2gm' not in sys.modules:  # Check it has not already been added (only required for Python2)
        # Import the submit2gm.command_port_server_utils module
        submit2gm_command_port_server_utils_path = os.path.join(submit2gm_package_path, "command_port_server_utils.py")

        import imp
        module = imp.load_source('submit2gm', submit2gm_init_path)
        sys.modules['submit2gm'] = module

        module = imp.load_source('submit2gm.command_port_server_utils', submit2gm_command_port_server_utils_path)
        sys.modules['submit2gm.command_port_server_utils'] = module


def open_gridmarkets_plugin(args):
    import_submit2gm()
    from submit2gm.command_port_server_utils import start_command_port_server
    start_command_port_server(ENVOY_PYTHON_PATH, LOGGER_NAME, PRODUCT, META_INFO, evaluate_code, DEBUG_MODE)


def print_debug_status(args):
    import_submit2gm()
    from submit2gm.command_port_server_utils import print_debug_status
    print_debug_status()


def raise_window(args):
    import_submit2gm()
    from submit2gm.command_port_server_utils import raise_window
    raise_window()


def stop_submit2gm(args):
    import_submit2gm()
    from submit2gm.command_port_server_utils import stop_submit2gm
    stop_submit2gm()


##########################################################
# Menu building code.
##########################################################

def __does_plugin_menu_item_exist():
    return maya.cmds.menu("MayaWindow|{}".format(MENU_NAME), parent="MayaWindow", exists=True)


def __add_actions_to_maya_menu_bar_item():
    maya.cmds.menuItem(label=MENU_ITEM_NAME,
                       image=GM_LOGO_PATH,
                       command=open_gridmarkets_plugin)

    if DEBUG_MODE:
        maya.cmds.menuItem(divider=True)
        maya.cmds.menuItem(label="Print debug status", command=print_debug_status)
        maya.cmds.menuItem(label="Raise window", command=raise_window)
        maya.cmds.menuItem(label="Stop Submit2gm", command=stop_submit2gm)


def __get_maya_major_version():
    return maya.cmds.about(version=True)


def __create_maya_menu_bar_btn():
    # If the menu bar item does not already exist create it
    if not __does_plugin_menu_item_exist():

        # create the menu
        maya.cmds.menu(MENU_NAME,
                       label=MENU_NAME,
                       tearOff=1,
                       parent='MayaWindow',
                       version=__get_maya_major_version())

        # add the actions to the menu
        __add_actions_to_maya_menu_bar_item()

        # highlight the new menu
        maya.cmds.whatsNewHighlight(highlightOn=True)


def __update_maya_menu_bar_actions():
    if __does_plugin_menu_item_exist():
        # remove all old items from the menu
        items = maya.cmds.menu("MayaWindow|{}".format(MENU_NAME), q=True, itemArray=True)
        for item in items:
            maya.cmds.deleteUI(item)

        # rebuild menu
        __add_actions_to_maya_menu_bar_item()


def __remove_maya_menu_bar_btn():
    # remove the menu bar item if it exists
    if __does_plugin_menu_item_exist():
        maya.cmds.deleteUI("MayaWindow|{}".format(MENU_NAME))
        maya.cmds.refresh()


def setup_plugin():
    # if the maya plugin menu item already exists
    if __does_plugin_menu_item_exist():
        # update its actions so that they are pointing to the new main window
        __update_maya_menu_bar_actions()
    else:
        # otherwise create the menu bar item
        __create_maya_menu_bar_btn()


##########################################################
# Plug-in initialization.
##########################################################

def maya_useNewAPI():
    """
    The presence of this function tells Maya that the plugin produces, and
    expects to be passed, objects created using the Maya Python API 2.0.
    """
    pass


def initializePlugin(mobject):
    ''' Initialize the plug-in when Maya loads it. '''

    # try to uninitialise the plug-in if it already exists
    try:
        uninitializePlugin(mobject)
    except ImportError as e:
        # An import error will be raised if the plug-in has not already been initialised
        pass

    mplugin = OpenMaya.MFnPlugin(mobject, vendor=VENDOR, version=VERSION_STR)

    # create and open the main plugin window
    setup_plugin()


def uninitializePlugin(mobject):
    ''' Uninitialize the plug-in when Maya un-loads it. '''
    mplugin = OpenMaya.MFnPlugin(mobject)
    __remove_maya_menu_bar_btn()
